<?php
/**
 * @version     5.0.1
 * @package     com_gausers
 * @copyright   Copyright (C) 2012. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Created by Glenn Arkell - http://www.glennarkell.com.au
 */

namespace GlennArkell\Component\Gausers\Administrator\Helper;

defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\Uri\Uri;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\Filesystem\Path;
use \Joomla\CMS\HTML\HTMLHelper;
use \GlennArkell\Component\Gausers\Administrator\Helper\GalistmembersPDFHelper;
use \GlennArkell\Component\Gausers\Administrator\Helper\GanamesHelper;

class GalistmembersHelper
{


	/**
	* Get current user records with partner and kids names
	* @return array of objects on success, boolean on fail
	*/
	public static function getCurrentMembers($params)
	{
        $inclGroup  = $params->get('sendto_group', 0);
        $exclMbrs  = $params->get('extract_excludes', array());
        $exMbrs = \implode(',', $exclMbrs);

        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
        $query->select(' id, SUBSTRING_INDEX(SUBSTRING_INDEX(name, \' \', 1), \' \', -1) AS first_name ');
        $query->select(' If( length(name) - length(replace(name, \' \', \'\'))>1, SUBSTRING_INDEX(SUBSTRING_INDEX(name, \' \', 2), \' \', -1) ,NULL) as middle1_name ');
        $query->select(' If( If( length(name) - length(replace(name, \' \', \'\'))>1, SUBSTRING_INDEX(SUBSTRING_INDEX(name, \' \', 3), \' \', -1) ,NULL) = SUBSTRING_INDEX(SUBSTRING_INDEX(name, \' \', 4), \' \', -1), null, If( length(name) - length(replace(name, \' \', \'\'))>1, SUBSTRING_INDEX(SUBSTRING_INDEX(name, \' \', 3), \' \', -1) ,NULL)) as middle2_name ');
        $query->select(' SUBSTRING_INDEX(SUBSTRING_INDEX(name, \' \', 4), \' \', -1) AS last_name ');
		$query->from(' #__users ');
        $query->where(' block = 0 ' );
        if (!empty($exclMbrs)) {
            $query->where(' id NOT IN ('.$exMbrs.')' );
        }
		$query->order(' last_name ASC ' );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

	}

	// * ------------------------   Creation of PDF   ----------------------------- * //
	/*
	 *  This is a mechanism to create a PDF list of members
	 *
	*/
	public static function createPDF()
	{
		Factory::getLanguage()->load('com_gausers', JPATH_ADMINISTRATOR);
		$params = ComponentHelper::getParams('com_gausers');
		$joint_mship  = $params->get('mship_single', 0);

        $members = self::getCurrentMembers($params);

		$lh = 5;
		$cntr = 0;

        //class instantiation
        //$pdf=new PDF("P","in","Letter"); // Legal Letter size
        //$pdf=new PDF("L","mm","A4");     // Landscape
        $pdf=new GalistmembersPDFHelper("P","mm","A4");

		$pdf->SetMargins(10,10,10);

        $pdf->AddPage();

        $pdf->SetFont('Arial','',10); // font-family, font-weight (B), font-size

        foreach ($members AS $m) {
            $mbr = GanamesHelper::breakdownNamesFromUserID($m->id);
            if ($joint_mship) {
                $name = GanamesHelper::combineNames($mbr);
            } else {
                $name = $mbr->last_name.', '.$mbr->first_name;
            }

            $cntr++;
            if ($cntr == 1) { $m1 = $name; continue; }
            if ($cntr == 2) {
                $cntr = 0;
                $pdf->Cell(10, $lh, "", 0, 0, "L"); // Sets an spacer
                $pdf->Cell(5, $lh, "", "TRBL", 0, "C");
                $pdf->Cell(10, $lh, "", 0, 0, "L"); // Sets an spacer
                $pdf->Cell(65, $lh, $m1, 0, 0, "L");
                $pdf->Cell(10, $lh, "", 0, 0, "L"); // Sets an spacer
                $pdf->Cell(10, $lh, "", 0, 0, "L"); // Sets an spacer
                $pdf->Cell(5, $lh, "", "TRBL", 0, "C");
                $pdf->Cell(10, $lh, "", 0, 0, "L"); // Sets an spacer
                $pdf->Cell(65, $lh, $name, 0, 1, "L");
                $pdf->Cell(190, 1, "", 0, 1, "L"); // Sets an spacer
            }
        }

        // clean up
        if ($cntr == 1) {
            $pdf->Cell(5, $lh, "", 0, 0, "L"); // Sets an spacer
            $pdf->Cell(5, $lh, "", "TRBL", 0, "L");
            $pdf->Cell(10, $lh, "", 0, 0, "L"); // Sets an spacer
            $pdf->Cell(40, $lh, $m1, 0, 0, "L");
        }

		$path = Path::clean( JPATH_SITE . '/images/members' );
        $pdf->Output($path . "/MembersList.pdf", "F");

		$attachfile = $path.'/MembersList.pdf';

        return $attachfile;

	}

}

